import os
import cmd
import random
import string
import tempfile
import argparse
import inspect

from subprocess import call

import zmq
from vexmessage import decode_vex_message

from vexbot import __version__
from vexbot.util import start_vexbot, create_vexdir
from vexbot.adapters.communication_messaging import ZmqMessaging


class Shell(cmd.Cmd):
    def __init__(self,
                 context=None,
                 prompt_name='vexbot',
                 publish_address=None,
                 subscribe_address=None,
                 **kwargs):

        super().__init__()
        # FIXME
        self.messaging = ZmqMessaging('command_line',
                                      publish_address,
                                      subscribe_address)

        self.stdout.write('Vexbot {}\n'.format(__version__))
        self.stdout.write('Type \"help\" for available commands\n')
        if kwargs.get('already_running', False):
            # TODO: add in version
            self.stdout.write('vexbot already running\n')

        self.messaging.register_command('start vexbot',
                                        start_vexbot)

        # TODO: allow the speficiation of a file suffix
        self.messaging.register_command('call editor',
                                        self._call_editor)

        # self.messaging.set_socket_identity('shell')
        self.messaging.set_socket_filter('')
        self.messaging.start_messaging()

        self.prompt = prompt_name + ': '
        self.misc_header = "Commands"

    def default(self, arg):
        if not self.messaging.is_command(arg):
            self.messaging.send_command(arg)
            if self.messaging.sub_socket.getsockopt(zmq.IDENTITY):
                frame = self.messaging.sub_socket.recv_multipart()
                msg = decode_vex_message(frame)
                self.stdout.write(msg)

    def _create_command_function(self, command):
        def resulting_function(arg):
            self.default(' '.join((command, arg)))
        return resulting_function

    def do_EOF(self, arg):
        self.stdout.write('\n')
        return True

    def get_names(self):
        return dir(self)

    def do_help(self, arg):
        if arg:
            # TODO
            pass
        else:
            self.stdout.write("{}\n".format(self.doc_leader))
            # TODO
            self.print_topics(self.misc_header,
                              ['start', 'restart', 'kill', 'killall',
                               'list', 'commands', 'alive', 'record',
                               'restartbot'],
                              15,
                              80)

    def add_completion(self, command):
        setattr(self,
                'do_{}'.format(command),
                self._create_command_function(command))

    def _call_editor(self):
        # TODO: Move into communication messaging, after creating auth
        vexdir = create_vexdir()
        code_output = _call_editor(vexdir)
        try:
            code = compile(code_output, '<string>', 'exec')
        except Exception as e:
            print(e)

        local = {}
        exec(code, globals(), local)
        # need to add to commands?
        for k, v in local.items():
            if inspect.isfunction(v):
                self.messaging.register_command(k, v)


def _call_editor(directory=None):
    editor = os.environ.get('EDITOR', 'vim')
    initial_message = b""
    file = None
    filename = None

    if directory is not None:
        # create a random filename
        filename = ''.join(random.choice(string.ascii_uppercase +
                                         string.digits) for _ in range(8))

        filename = filename + '.py'
        # TODO: loop back if isfile is True
        filepath = os.path.join(directory, filename)
        file = open(filepath, 'w+b')
    else:
        file = tempfile.NamedTemporaryFile(prefix=directory, suffix='.py')
        filename = file.name
    current_dir = os.getcwd()
    os.chdir(directory)
    file.write(initial_message)
    file.flush()
    call([editor, filename])
    file.close()
    file = open(filename)
    message = file.read()
    file.close()
    os.chdir(current_dir)

    return message


def _get_kwargs():
    parser = argparse.ArgumentParser()
    parser.add_argument('--publish_address', default=None)
    parser.add_argument('--prompt_name', default='vexbot')

    args = parser.parse_args()
    return vars(args)


def main(**kwargs):
    if not kwargs:
        kwargs = _get_kwargs()
    shell = Shell(**kwargs)
    shell.cmdloop()

if __name__ == '__main__':
    main()



import zmq
from vexmessage import create_vex_message


class ZmqMessaging:
    def __init__(self,
                 service_name,
                 pub_address=None,
                 sub_address=None,
                 socket_filter=None):

        self.pub_socket = None
        self.sub_socket = None

        self._service_name = service_name
        self._pub_address = pub_address
        self._sub_address = sub_address
        self._socket_filter = socket_filter

        self._messaging_started = False
        self._commands = {}
        try:
            import setproctitle
            setproctitle.setproctitle(service_name)
        except ImportError:
            pass

    def start_messaging(self):
        if self._messaging_started:
            self.update_messaging()
            return

        context = zmq.Context()
        self.pub_socket = context.socket(zmq.PUB)
        self.sub_socket = context.socket(zmq.SUB)
        if self._pub_address:
            try:
                self.pub_socket.connect(self._pub_address)
            except zmq.error.ZMQError:
                err = """
                      Incorrect value passed in for socket address in {}, fix
                      it in your settings.yml or default_settings.yml
                      """.format(self._service_name)

                print(err)

            if self._socket_filter is not None:
                self.set_socket_filter(self._socket_filter)

        if self._sub_address:
            self.sub_socket.connect(self._sub_address)

        self._messaging_started = True

    def update_messaging(self):
        if self._pub_address:
            self.pub_socket.bind(self._pub_address)
        if self._sub_address:
            self.sub_socket.bind(self._sub_address)

    def set_socket_filter(self, filter_):
        if self.sub_socket:
            self.sub_socket.set_string(zmq.SUBSCRIBE, filter_)
        else:
            self._socket_filter = filter_

    def send_message(self, *msg, target=''):
        frame = create_vex_message(target, self._service_name, 'MSG', msg)
        self.pub_socket.send_multipart(frame)

    def send_status(self, *status, target=''):
        frame = create_vex_message(target,
                                   self._service_name,
                                   'STATUS',
                                   status)

        self.pub_socket.send_multipart(frame)

    def send_command(self, *command, target=''):
        frame = create_vex_message(target, self._service_name, 'CMD', command)
        self.pub_socket.send_multipart(frame)

    def register_command(self, cmd, function):
        self._commands[cmd] = function

    def is_command(self, cmd, call_command=True):
        callback = self._commands.get(cmd, None)

        if callback and call_command:
            callback_result = callback()
            if callback_result:
                self.send_message(callback_result)

            return True
        elif callback:
            return True

        return False



import os
import sys
import string
from subprocess import Popen
from vexbot.util import _get_config


# need a way to input the code |solved for shell input case|
# need to store code permenantly |need to metadata to this|
# need to load code, w/ metadata

# compile function allows us to compile 'single' and 'exec'

# Create error catching
# feeback loop
#  Need to implement pub-sub filtering |CHECK THE BOX|

# need to store the code in a file
# need to load the code somehow
# run the code

INDENTCHARS = string.ascii_letters + string.digits + '_'

class CommandManager:
    # Think about passing in an adapter? And hooking up a command manager to
    # adapter
    def __init__(self, robot):
        self.r = robot
        self._commands = ['start', 'restart', 'kill', 'killall',
                          'list', 'alive']

        self.indentchars = INDENTCHARS

    def parse_commands(self, command):
        command = command.strip()
        if not command:
            return
        i, n = 0, len(command)
        while i < n and command[i] in self.indentchars: i = i + 1
        command, arg = command[:i], command[i:].strip()
        commands = arg.split()
        if command == 'start':
            self.r.subprocess_manager.start(commands)
        elif command == 'restart':
            self.r.subprocess_manager.restart(commands)
        elif command == 'kill':
            self.r.subprocess_manager.kill(commands)
        elif command == 'killall':
            self.r.subprocess_manager.killall()
            sys.exit()
        elif command == 'help':
            print(self._commands)
            print('vexbot: ', end=None)
        elif command == 'list':
            lists = [x for
                     x in
                     self.r.subprocess_manager.registered_subprocesses()]

            if lists:
                print(*lists)
                print('vexbot: ', end=None)
        elif command == 'commands':
            print(*self._commands)
            print('vexbot: ', end=None)
        elif command == 'alive':
            values = self.r.subprocess_manager.registered_subprocesses()
            for v in values:
                self.r.messaging.send_message('MSG', 'ben', 'alive', target=v)
        elif command == 'record':
            self.r.messaging.send_command(('record',))
        elif command == 'restartbot':
            config = _get_config()
            settings_path = config.get('settings_path')
            directory = os.path.abspath(os.path.dirname(__file__))
            robot = os.path.join(directory, 'robot.py')
            args = (sys.executable,
                    '-c',
                    "import time\nimport sys\nfrom subprocess import Popen\ntime.sleep(1)\nPopen((sys.executable, '{}', '--settings_path', '{}'))".format(robot, settings_path))

            Popen(args)
            sys.exit()



import logging
import argparse
from sleekxmpp import ClientXMPP
from sleekxmpp.exceptions import IqError, IqTimeout
from time import sleep

import zmq

from vexbot.adapters.communication_messaging import ZmqMessaging # flake8: noqa


class ReadOnlyXMPPBot(ClientXMPP):
    def __init__(self,
                 jid,
                 password,
                 room,
                 socket_address,
                 service_name,
                 bot_nick='EchoBot',
                 **kwargs):

        # Initialize the parent class
        super().__init__(jid, password)
        self.messaging = ZmqMessaging(service_name, socket_address)
        self.messaging.set_socket_filter('')
        self.messaging.start_messaging()

        self.room = room
        self.nick = bot_nick
        self.log = logging.getLogger(__file__)

        # One-shot helper method used to register all the plugins
        self._register_plugin_helper()

        self.add_event_handler("session_start", self.start)
        self.add_event_handler("groupchat_message", self.muc_message)
        self.add_event_handler('connected', self._connected)
        self.add_event_handler('disconnected', self._disconnected)

    def _disconnected(self, *args):
        self.messaging.send_status('DISCONNECTED')

    def _connected(self, *args):
        self.messaging.send_status('CONNECTED')

    def _register_plugin_helper(self):
        """
        One-shot helper method used to register all the plugins
        """
        # Service Discovery
        self.register_plugin('xep_0030')
        # XMPP Ping
        self.register_plugin('xep_0199')
        # Multiple User Chatroom
        self.register_plugin('xep_0045')

    def start(self, event):
        self.log.info('starting xmpp')
        self.send_presence()
        self.plugin['xep_0045'].joinMUC(self.room,
                                        self.nick,
                                        wait=True)

        self.get_roster()

    def muc_message(self, msg):
        self.messaging.send_message(msg['mucnick'], msg['body'])


def _get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--local', help='local arg for string parsing')
    parser.add_argument('--domain', help='domain for xmpp')
    parser.add_argument('--room', help='room!')
    parser.add_argument('--resource', help='resource')
    parser.add_argument('--password', help='password')
    parser.add_argument('--service_name')
    parser.add_argument('--socket_address')
    parser.add_argument('--bot_nick')

    return parser.parse_args()


def main():
    args = _get_args()
    jid = '{}@{}/{}'.format(args.local, args.domain, args.resource)
    kwargs = vars(args)
    already_running = False

    try:
        xmpp_bot = ReadOnlyXMPPBot(jid, **kwargs)
    except zmq.ZMQError:
        already_running = True

    if not already_running:
        while True:
            try:
                xmpp_bot.connect()
                xmpp_bot.process(block=True)
            except Exception as e:
                xmpp_bot.log.error(e)

if __name__ == '__main__':
    main()



import sys
import atexit
import signal
from subprocess import Popen, DEVNULL


class SubprocessManager:
    def __init__(self):
        # this is going to be a list of filepaths
        self._registered = {}
        # these will be subprocesses
        self._subprocess = {}
        atexit.register(self._close_subprocesses)
        signal.signal(signal.SIGINT, self._handle_close_signal)
        signal.signal(signal.SIGTERM, self._handle_close_signal)

    def _handle_close_signal(self, signum=None, frame=None):
        self._close_subprocesses()
        sys.exit()

    def _close_subprocesses(self):
        """
        signum and frame are part of the signal lib
        """
        for process in self._subprocess.values():
            process.terminate()

    def registered_subprocesses(self):
        return self._registered.keys()

    def register(self, keys, values, settings=None):
        if settings is None:
            settings = [None for _ in range(len(keys))]

        for key, value, setting in zip(keys, values, settings):
            self._registered[key] = (value, setting)

    def update(self, key, value=None, setting=None):
        """
        This allows us to update the settings or value
        """
        if setting is None:
            setting = self._registered[key][1]
        if value is None:
            value = self._registered[key][0]
        self._registered[key] = (value, setting)

    def start(self, keys):
        for key in keys:
            try:
                data = self._registered[key]
            except KeyError:
                data = None

            if data:
                args = [sys.executable, data[0]]
                if data[1]:
                    args.extend(data[1])
                process = Popen(args, stdout=DEVNULL)
                self._subprocess[key] = process

    def restart(self, values):
        for value in values:
            process = self._subprocess[value]
            process.kill()
            self.start(value)

    def kill(self, values):
        for value in values:
            try:
                process = self._subprocess[value]
                process.kill()
            except KeyError:
                pass

    def killall(self):
        for subprocess in self._subprocess.values():
            subprocess.kill()



import sys
import asyncio
import argparse

import irc3
import zmq
from zmq import ZMQError
from irc3.plugins.autojoins import AutoJoins

from vexbot.adapters.communication_messaging import ZmqMessaging


@irc3.plugin
class AutoJoinMessage(AutoJoins):
    requires = ['irc3.plugins.core', ]

    def __init__(self, bot):
        super().__init__(bot)

    def connection_lost(self):
        self.bot.messaging.send_status('DISCONNECTED')
        super(AutoJoinMessage, self).connection_lost()

    def join(self, channel=None):
        super(AutoJoinMessage, self).join(channel)
        self.bot.messaging.send_status('CONNECTED')

    @irc3.event(irc3.rfc.KICK)
    def on_kick(self, mask, channel, target, **kwargs):
        self.bot.messaging.send_status('DISCONNECTED')
        super().on_kick(mask, channel, target, **kwargs)

    @irc3.event("^:\S+ 47[1234567] \S+ (?P<channel>\S+).*")
    def on_err_join(self, channel, **kwargs):
        self.bot.messaging.send_status('DISCONNECTED')
        super().on_err_join(channel, **kwargs)


@irc3.plugin
class EchoToMessage(object):
    requires = ['irc3.plugins.core',
                'irc3.plugins.command']

    def __init__(self, bot):
        self.bot = bot

    @irc3.event(irc3.rfc.PRIVMSG)
    def message(self, mask, event, target, data):
        nick = mask.split('!')[0]
        message = data
        self.bot.messaging.send_message(nick, str(message))


def create_irc_bot(nick,
                   password,
                   host=None,
                   port=6667,
                   realname=None,
                   channel=None):

    config = dict(ssl=False,
                  includes=['irc3.plugins.core',
                            'irc3.plugins.command',
                            'irc3.plugins.human',
                            __name__])

    if realname is None:
        realname = nick
    if channel is None:
        channel = nick

    config['nick'] = nick
    config['password'] = password
    config['host'] = host
    config['port'] = port
    config['username'] = realname
    config['autojoins'] = channel
    config['level'] = 30

    bot = irc3.IrcBot.from_config(config)

    return bot


async def _check_subscription(bot):
    while True:
        await asyncio.sleep(1)
        msg = None
        try:
            msg = bot.messaging.sub_socket.recv_multipart(zmq.NOBLOCK)
        except zmq.error.Again:
            pass
        if msg:
            print(msg)


def main(nick, password, host, channel, socket_address, service_name):
    irc_client = create_irc_bot(nick,
                                password,
                                host,
                                channel=channel)

    try:
        messaging = ZmqMessaging(service_name, socket_address)
        messaging.set_socket_filter('irc')
        messaging.start_messaging()
    except ZMQError:
        return
    # Duck type messaging onto irc_client, FTW
    irc_client.messaging = messaging

    irc_client.create_connection()
    irc_client.add_signal_handlers()
    event_loop = asyncio.get_event_loop()
    asyncio.ensure_future(_check_subscription(irc_client))
    try:
        event_loop.run_forever()
    except KeyboardInterrupt:
        pass
    event_loop.close()
    sys.exit()


def _get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--nick')
    parser.add_argument('--password')
    parser.add_argument('--host')
    parser.add_argument('--channel')
    parser.add_argument('--socket_address')
    parser.add_argument('--service_name')

    return parser.parse_args()


if __name__ == '__main__':

    kwargs = vars(_get_args())
    main(**kwargs)



import argparse
import logging
import signal
from time import sleep

import selenium
from selenium import webdriver
from zmq import ZMQError

from vexbot.adapters.communication_messaging import ZmqMessaging # flake8: noqa


class JavascriptWebscraper:
    def __init__(self,
                 url=None,
                 comment_element_id=None,
                 author_class_name=None,
                 message_class_name=None,
                 socket_address='',
                 service_name=''):

        """
        `comment_element_id` is the css element where all the comments are,
        i.e., 'all-comments' for youtube

        `author_class_name` is the css class which holds the comment author
        username i.e., 'yt-user-name' for youtube

        `message_class_name` is the css class which holds the comment test
        ie., 'comment-text' for youtube
        """
        self.messaging = ZmqMessaging(service_name, socket_address)
        messaging.set_socket_filter('')
        self.messaging.start_messaging()
        self.log = logging.getLogger(__name__)
        self.log.setLevel(logging.NOTSET)

        self.url = url
        self._number_of_messages = 0

        self.comment_element_id = comment_element_id
        self.author_class_name = author_class_name
        self.message_class_name = message_class_name
        self._driver = None
        self._kill = False
        signal.signal(signal.SIGINT, self._exit_gracefully)
        signal.signal(signal.SIGTERM, self._exit_gracefully)

    def _exit_gracefully(self, *args, **kwargs):
        if self._driver is not None:
            self._driver.quit()
        self._kill = True

    def run_forever(self):
        while True:
            try:
                self.log.info('Starting javascript scraper!')
                self.run()
            except selenium.common.exceptions.NoSuchElementException:
                self.log.error('Youtube parameters wrong, shutting down :(')
                break
            except Exception as e:
                if self._kill:
                    break
                else:
                    self.log.exception('Javascript error!', e)

    def run(self):
        if self._driver:
            self._driver.quit()
            self._driver = None
        self.log.info('starting up phantom javascript!')
        self._driver = webdriver.PhantomJS()
        # TODO: see if this is needed or not
        self._driver.set_window_size(1000, 1000)
        self._driver.get(self.url)

        # NOTE: need some time for comments to load
        self.log.info('youtube sleeping for 5 seconds!')
        sleep(5)
        self.log.info('youtube done sleeping')

        all_comments = self._driver.find_element_by_id(self.comment_element_id)
        # TODO: add in a signal here that all is connected!

        # NOTE: make sure this is ok if using for anything other than youtube
        comments = all_comments.find_elements_by_tag_name('li')
        self._number_of_messages = len(comments)
        self.messaging.send_status('CONNECTED')

        while True:
            sleep(1)
            comments = all_comments.find_elements_by_tag_name('li')
            comments_length = len(comments)

            if comments_length > self._number_of_messages:
                # NOTE: this number is intentionally NEGATIVE
                msgs_not_parsed = self._number_of_messages - comments_length

                self._number_of_messages = len(comments)
                comments = comments[msgs_not_parsed:]
                for comment in comments:
                    find_elem = comment.find_element_by_class_name
                    author = find_elem(self.author_class_name).text

                    message = find_elem(self.message_class_name).text
                    self.messaging.send_message(author, message)

        self.messaging.send_status('DISCONNECTED')

def _get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--url')
    parser.add_argument('--comment_element_id')
    parser.add_argument('--author_class_name')
    parser.add_argument('--message_class_name')
    parser.add_argument('--socket_address')
    parser.add_argument('--service_name')

    return parser.parse_args()


if __name__ == '__main__':
    # python magic to get a list of args
    # vars changes namespace to dict, `values()` gets the values out of dict
    kwargs = vars(_get_args())
    already_running = False
    try:
        webscraper = JavascriptWebscraper(**kwargs)
    except ZMQError:
        already_running = True

    if not already_running:
        webscraper.run_forever()



import argparse
import json
import requests
import html
import logging
from time import sleep

import websocket
from zmq import ZMQError

from vexbot.adapters.communication_messaging import ZmqMessaging


class ReadOnlyWebSocket(websocket.WebSocketApp):
    # NOTE: chat_signal defined in `__init__`

    def __init__(self,
                 streamer_name,
                 namespace,
                 website_url,
                 socket_address,
                 service_name):

        self.log = logging.getLogger(__name__)
        self.log.setLevel(0)
        self.messaging = ZmqMessaging(service_name, socket_address)
        self.messaging.start_messaging()
        self.messaging.set_socket_filter('')
        self._streamer_name = streamer_name
        self.namespace = namespace
        self._website_url = website_url
        self.log.info('Getting Socket IO key!')
        self.key, heartbeat = self._connect_to_server_helper()
        self.log.info('Socket IO key got!')

        # alters URL to be more websocket...ie
        self._website_socket = self._website_url.replace('http', 'ws')
        self._website_socket += 'websocket/'
        super().__init__(self._website_socket + self.key,
                         on_open=self.on_open,
                         on_close=self.on_close,
                         on_message=self.on_message,
                         on_error=self.on_error)

    def repeat_run_forever(self):
        while True:
            try:
                self.run_forever()
            except KeyboardInterrupt:
                break
            except Exception as e:
                self.log.info('Socket IO errors: {}'.format(e))
            sleep(3)
            self.messaging.send_status('DISCONNECTED')
            key, _ = self._connect_to_server_helper()
            self.url = self._website_socket + key

    def _connect_to_server_helper(self):
        r = requests.post(self._website_url)
        params = r.text

        # unused variables are connection_timeout and supported_formats
        key, heartbeat_timeout, _, _ = params.split(':')
        heartbeat_timeout = int(heartbeat_timeout)
        return key, heartbeat_timeout

    def on_open(self, *args):
        logging.info('Websocket open!')

    def on_close(self, *args):
        logging.info('Websocket closed :(')

    def on_message(self, *args):
        message = args[1].split(':', 3)
        key = int(message[0])
        # namespace = message[2]

        if len(message) >= 4:
            data = message[3]
        else:
            data = ''
        if key == 1 and args[1] == '1::':
            self.send_packet_helper(1)
        elif key == 1 and args[1] == '1::{}'.format(self.namespace):
            self.send_packet_helper(5, data={'name': 'initialize'})
            data = {'name': 'join',
                    'args': ['{}'.format(self._streamer_name)]}

            self.send_packet_helper(5, data=data)
            self.log.info('Connected to channel with socket io!')
            self.messaging.send_status('CONNECTED')
        elif key == 2:
            self.send_packet_helper(2)
        elif key == 5:
            data = json.loads(data, )
            if data['name'] == 'message':
                message = data['args'][0]
                sender = html.unescape(message['sender'])
                message = html.unescape(message['text'])
                self.messaging.send_message(sender, message)

    def on_error(self, *args):
        print(args[1])

    def disconnect(self):
        callback = ''
        data = ''
        # '1::namespace'
        self.send(':'.join([str(self.TYPE_KEYS['DISCONNECT']),
                           callback, self.namespace, data]))

    def send_packet_helper(self,
                           type_key,
                           data=None):

        if data is None:
            data = ''
        else:
            data = json.dumps(data)

        # NOTE: callbacks currently not implemented
        callback = ''
        message = ':'.join([str(type_key), callback, self.namespace, data])
        self.send(message)


def _get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--streamer_name')
    parser.add_argument('--namespace')
    parser.add_argument('--website_url')
    parser.add_argument('--service_name')
    parser.add_argument('--socket_address')

    return parser.parse_args()


if __name__ == '__main__':

    args = vars(_get_args())
    already_running = False
    try:
        client = ReadOnlyWebSocket(**args)
    except ZMQError:
        already_running = True

    if not already_running:
        client.repeat_run_forever()
